package com.gic.enterprise.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.dao.mapper.TabAuditLogMapper;
import com.gic.enterprise.dto.AuditLogDTO;
import com.gic.enterprise.entity.TabAuditLog;
import com.gic.enterprise.qo.AuditLogQO;
import com.gic.enterprise.service.AuditLogService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-10-25 15:32
 */
@Service
public class AuditLogServiceImpl implements AuditLogService {

    @Autowired
    private TabAuditLogMapper tabAuditLogMapper;

    @Override
    public TabAuditLog getLastWaitAuditByApplyUser(Integer enterpriseId, Integer applyUserId, Integer projectItemId) {
        AuditLogDTO auditLogDTO = new AuditLogDTO();
        auditLogDTO.setEnterpriseId(enterpriseId);
        auditLogDTO.setApplyUserId(applyUserId);
        auditLogDTO.setProjectItemId(projectItemId);
        auditLogDTO.setAuditExpireTime(new Date());
        return tabAuditLogMapper.getLastWaitAuditByApplyUser(auditLogDTO);
    }

    @Override
    public void update(TabAuditLog auditLog) {
        TabAuditLog result = new TabAuditLog();
        result.setAuditExpireTime(new Date());
        result.setAuditLogId(auditLog.getAuditLogId());
        tabAuditLogMapper.updateByPrimaryKeySelective(result);
    }

    @Override
    public Integer save(AuditLogDTO auditLogDTO) {
        TabAuditLog auditLog = EntityUtil.changeEntityByJSON(TabAuditLog.class, auditLogDTO);
        auditLog.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        auditLog.setCreateTime(new Date());
        tabAuditLogMapper.insertSelective(auditLog);
        return auditLog.getAuditLogId();
    }

    @Override
    public TabAuditLog getAuditById(Integer auditLogId) {
        return tabAuditLogMapper.selectByPrimaryKey(auditLogId);
    }

    @Override
    public void updateAudit(AuditLogDTO auditLogDTO) {
        Date date = new Date();
        TabAuditLog auditLog = new TabAuditLog();
        auditLog.setAuditLogId(auditLogDTO.getAuditLogId());
        auditLog.setAuditorId(auditLogDTO.getAuditorId());
        auditLog.setAuditorName(auditLogDTO.getAuditorName());
        auditLog.setAuditorPhone(auditLogDTO.getAuditorPhone());
        auditLog.setAuditTime(date);
        auditLog.setAuditResult(auditLogDTO.getAuditResult());
        auditLog.setUpdateTime(date);
        auditLog.setAuditReason(auditLogDTO.getAuditReason());
        tabAuditLogMapper.updateByPrimaryKeySelective(auditLog);
    }

    @Override
    public Page<TabAuditLog> listAuditLog(AuditLogQO auditLogQO) {
        PageHelper.startPage(auditLogQO.getCurrentPage(), auditLogQO.getPageSize());
        return tabAuditLogMapper.listAuditLog(auditLogQO);
    }

    @Override
    public boolean hasProjectItemInUse(Integer projectItemId) {
        return tabAuditLogMapper.getWaitAuditByProjectItem(projectItemId, new Date()) != null;
    }
}
