package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.wm.WmStoreSyncLogDTO;
import com.gic.enterprise.entity.TabWmStoreSyncLog;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.WmStoreSyncLogApiService;
import com.gic.enterprise.service.WmStoreSyncLogService;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.service.StoreApiService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

@Service("wmStoreSyncLogApiService")
public class WmStoreSyncLogApiServiceImpl implements WmStoreSyncLogApiService {
    @Autowired
    private WmStoreSyncLogService wmStoreSyncLogService;
    @Autowired
    private StoreApiService storeApiService;

    @Override
    public ServiceResponse<Void> saveWmStoreSyncLog(List<Integer> storeIdList, Integer wmMallStoreId, Integer type,
            Integer enterpriseId) {
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        StringBuilder storeIds = new StringBuilder();
        storeIdList.forEach(storeId -> storeIds.append(storeId).append(","));
        storeSearchDTO.setStoreIds(storeIds.toString());
        ServiceResponse<Page<StoreDTO>> storeResponse = storeApiService.listStore(storeSearchDTO, 0, storeIdList.size());
        if (storeResponse.isSuccess()) {
            Page<StoreDTO> page = storeResponse.getResult();
            if (page != null) {
                List<StoreDTO> storeList = page.getResult();
                if (CollectionUtils.isNotEmpty(storeList)) {
                    storeList.forEach(temp -> wmStoreSyncLogService
                            .saveWmStoreSyncLog(temp.getStoreId(), enterpriseId, wmMallStoreId, type, temp.getStoreCode(), temp.getStoreName()));
                }
                return ServiceResponse.success();
            }
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "ES没有门店数据");
    }

    @Override
    public ServiceResponse<Page<WmStoreSyncLogDTO>> pageWmStoreSyncLog(String search, Integer type,
                                                                       Integer wmMallStoreId, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page page = wmStoreSyncLogService.pageWmStoreSyncLog(search, type, wmMallStoreId, pageNum, pageSize);
        Page<WmStoreSyncLogDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, WmStoreSyncLogDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<List<WmStoreSyncLogDTO>> getStoreList(List<Integer> type, Integer wmMallStoreId) {
        List<TabWmStoreSyncLog> list = wmStoreSyncLogService.getStoreList(type, wmMallStoreId);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(WmStoreSyncLogDTO.class, list));
        }
        return ServiceResponse.success(Collections.emptyList());
    }
}
