package com.gic.enterprise.web.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collector;
import java.util.stream.Collectors;

import com.gic.api.base.commons.Page;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.dto.BillingPayInfoDTO;
import com.gic.enterprise.utils.UserDetail;
import com.gic.finance.constant.TransferAccountApprovalStatusEnum;
import com.gic.finance.dto.TransferAccountsApprovalDTO;
import com.gic.finance.service.TransferAccountsApprovalApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.bizdict.api.dto.BizdictDTO;
import com.gic.bizdict.api.service.BizdictService;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.constant.billing.InvoiceStatusEnum;
import com.gic.enterprise.dto.InvoiceDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.BillListQueryQO;
import com.gic.enterprise.qo.InvoiceListQueryQO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.BillingPayInfoApiService;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.BankInfoVO;
import com.gic.enterprise.web.vo.BillListVO;
import com.gic.enterprise.web.vo.billing.InvoiceListVO;
import com.gic.finance.constant.EnterprisePlatformTypeEnum;
import com.gic.finance.dto.InvoiceManageDTO;
import com.gic.finance.qo.InvoiceManageListQueryQO;
import com.gic.finance.service.InvoiceManageApiService;
import com.gic.redis.data.util.RedisUtil;

@RestController
@RequestMapping("/billing-pay-info")
public class BillingPayInfoController {
    private static final Logger LOGGER = LoggerFactory.getLogger(BillingPayInfoController.class);

    @Autowired
    private BillingPayInfoApiService billingPayInfoApiService;
    @Autowired
    private InvoiceManageApiService invoiceManageApiService;
    @Autowired
    private BizdictService bizdictService;
    @Autowired
    private TransferAccountsApprovalApiService transferAccountsApprovalApiService;

    @RequestMapping("/bank-info")
    public RestResponse bankInfo() {
        BizdictDTO dict = bizdictService.getByCode(Constants.BANK_ACCOUNT, "1");
        BankInfoVO bankInfoVO = new BankInfoVO();
        bankInfoVO.setCompanyName(dict.getName());
        bankInfoVO.setBankName(dict.getAliasName());
        bankInfoVO.setAccount(dict.getValueContent());
        String code = CreateRandomUtils.getStringRandom(4);
        RedisUtil.setCache("enterprise:billing:verificationcode:" + UserDetailUtils.getUserDetail().getEnterpriseId(),
                code, 300L);
        bankInfoVO.setVerificationCode(code);
        return RestResponse.success(bankInfoVO);
    }

    /**
     * 为开票单据列表
     * @Title: listBill  
     * @Description:
      * @author guojuxing
     * @param params 
     * @return com.gic.commons.webapi.reponse.RestResponse  
      */
    @RequestMapping("/list-bill")
    public RestResponse listBill(BillListQueryQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Page<BillingPayInfoDTO>> payInfoResponse = billingPayInfoApiService.listBill(params);
        if (payInfoResponse.isSuccess()) {
            Page<BillingPayInfoDTO> page = payInfoResponse.getResult();
            if (page != null) {
                List<BillingPayInfoDTO> list = page.getResult();
                if (CollectionUtils.isNotEmpty(list)) {
                    //银行对公转账，需要查询坏账金额
                    List<String> orderSerialNumberList = list.stream()
                            .filter(e -> e.getPayType() == 3)
                            .map(e -> e.getOrderSerialNumber()).collect(Collectors.toList());
                    final Map<String, Object> badMoneyMap;
                    ServiceResponse<List<TransferAccountsApprovalDTO>> approvalResponse = transferAccountsApprovalApiService
                            .listByOrderSerialNumber(orderSerialNumberList);
                    if (approvalResponse.isSuccess()) {
                        badMoneyMap = approvalResponse.getResult().stream().filter(e -> e.getApprovalStatus() == TransferAccountApprovalStatusEnum.AGREE.getCode())
                                .collect(Collectors.toMap(e -> e.getOrderNumber(), e -> e.getBadAmount(), (oldValue,newValue) -> newValue));
                    } else {
                        badMoneyMap = new HashMap<>(2);
                    }
                    Page<BillListVO> resultPage = EntityUtil.changeEntityNew(Page.class, page);
                    List<BillListVO> voList = list.stream()
                            .map(e -> {
                                BillListVO vo = EntityUtil.changeEntityNew(BillListVO.class, e);
                                vo.setBadAmount((Double) badMoneyMap.get(e.getOrderSerialNumber()));
                                return vo;
                            })
                            .collect(Collectors.toList());
                    resultPage.setResult(voList);
                    return RestResponse.success(resultPage);
                }
            }
        }
        return RestResponse.failure(payInfoResponse.getCode(), payInfoResponse.getMessage());
    }

    @RequestMapping("/list-pay-type")
    public RestResponse listPayType() {
        Map<Integer, String> resultMap = new HashMap<>(16);
        for (PayTypeEnum payTypeEnum : PayTypeEnum.values()) {
            if (payTypeEnum.getCode() == PayTypeEnum.ALI_PAY.getCode()) {
                //不展示支付宝
                continue;
            }
            resultMap.put(payTypeEnum.getCode(), payTypeEnum.getMessage());
        }
        return RestResponse.success(resultMap);
    }

    @RequestMapping("/list-invoice-status")
    public RestResponse listInvoiceStatus() {
        Map<Integer, String> resultMap = new HashMap<>(16);
        for (InvoiceStatusEnum invoiceStatusEnum : InvoiceStatusEnum.values()) {
            resultMap.put(invoiceStatusEnum.getCode(), invoiceStatusEnum.getMessage());
        }
        return RestResponse.success(resultMap);
    }

    /**
     * 开票
     * @Title: applyInvoice  
     * @Description:
      * @author guojuxing
     * @param invoiceDTO
     * @param billingAmount
     * @param invoiceType 
     * @return com.gic.commons.webapi.reponse.RestResponse  
      */
    @RequestMapping("/apply-invoice")
    public RestResponse applyInvoice(InvoiceDTO invoiceDTO, Double billingAmount, Integer invoiceType,
            String serialNumbers) {
        //验证流水号参数
        if (StringUtils.isBlank(serialNumbers)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "流水号不能为空");
        }
        String[] serialNumberArr;
        try {
            serialNumberArr = serialNumbers.split(",");
            LOGGER.info("开票流水号参数:{}", serialNumberArr);
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "流水号数据格式有误");
        }
        for (String serial : serialNumberArr) {
            ServiceResponse serialResponse = billingPayInfoApiService.getPayInfoBySerialNumber(serial);
            if (!serialResponse.isSuccess()) {
                return EnterpriseRestResponse.failure(serialResponse);
            }
        }
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        Integer enterpriseId = userDetail.getEnterpriseId();
        ServiceResponse invoiceInfo = billingPayInfoApiService.saveInvoiceAccountToRedis(invoiceDTO, enterpriseId);
        if (invoiceInfo.isSuccess()) {
            InvoiceManageDTO params = new InvoiceManageDTO();
            params.setPlatformType(EnterprisePlatformTypeEnum.GIC.getCode());
            params.setInvoiceType(invoiceType);
            params.setInitiator(UserDetailUtils.getUserDetail().getUserId().toString());
            params.setBillingAmount(billingAmount);
            params.setEnterpriseId(enterpriseId);
            params.setEnterpriseName(userDetail.getEnterpriseInfo().getEnterpriseName());
            params.setCompanyName(userDetail.getEnterpriseInfo().getCompanyName());
            params.setLogo(userDetail.getEnterpriseInfo().getLogo());
            params.setTaxNumber(invoiceDTO.getTaxNumber());
            params.setAccountName(invoiceDTO.getAccountName());
            params.setAddress(invoiceDTO.getAddress());
            params.setBank(invoiceDTO.getBank());
            params.setAccountPhone(invoiceDTO.getAccountPhone());
            params.setBankAccount(invoiceDTO.getBankAccount());
            params.setShippingName(invoiceDTO.getShippingName());
            params.setShippingAddress(invoiceDTO.getShippingAddress());
            params.setShippingPhone(invoiceDTO.getShippingPhone());
            ServiceResponse invoice = invoiceManageApiService.applyInvoice(params);
            if (invoice.isSuccess()) {
                ServiceResponse updateInvoiceStatus = billingPayInfoApiService.updateInvoiceStatus(serialNumbers);
                if (updateInvoiceStatus.isSuccess()) {
                    LogUtils.createLog("申请开票", "发票管理");
                    return RestResponse.success();
                } else {
                    return EnterpriseRestResponse.failure(updateInvoiceStatus);
                }
            } else {
                return EnterpriseRestResponse.failure(invoice);
            }
        } else {
            return EnterpriseRestResponse.failure(invoiceInfo);
        }
    }

    @RequestMapping("/get-invoice-info")
    public RestResponse getInvoiceInfo() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonResult(billingPayInfoApiService.getInvoiceInfoFromRedis(enterpriseId));
    }

    /**
     * 取消开票申请
     * @Title: cancelInvoice 
     * @Description:
      * @author guojuxing
     * @param invoiceManageId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/cancel-invoice")
    public RestResponse cancelInvoice(Integer invoiceManageId) {
        ServiceResponse<String> result = invoiceManageApiService.cancelInvoice(invoiceManageId);
        return OperationResultUtils.operationResult(result, "取消开票", OperationResultUtils.getOperationObject(result));
    }

    /**
     * 已申请开票单据列表
     * @Title: listInvoice  
     * @Description:
      * @author guojuxing
     * @param params 
     * @return com.gic.commons.webapi.reponse.RestResponse  
      */
    @RequestMapping("/list-invoice")
    public RestResponse listInvoice(InvoiceListQueryQO params) {
        InvoiceManageListQueryQO paramQO = new InvoiceManageListQueryQO();
        paramQO.setEndTime(params.getEndTime());
        paramQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        paramQO.setStartTime(params.getStartTime());
        paramQO.setInvoiceType(params.getInvoiceType());
        if (params.getInvoiceStatus() != null) {
            int invoiceStatus = params.getInvoiceStatus().intValue();
            if (InvoiceStatusEnum.APPLYING.getCode() == invoiceStatus) {
                //如果是申请中，则对应的是待审核&待邮寄
                paramQO.setInvoiceStatusStr(com.gic.finance.constant.InvoiceStatusEnum.TO_BE_ISSUED.getCode() + ","
                        + com.gic.finance.constant.InvoiceStatusEnum.TO_BE_MAILED.getCode());
            }
            if (InvoiceStatusEnum.CANCEL.getCode() == invoiceStatus) {
                paramQO.setInvoiceStatusStr(com.gic.finance.constant.InvoiceStatusEnum.CANCEL.getCode() + "");
            }
            if (InvoiceStatusEnum.REJECT.getCode() == invoiceStatus) {
                paramQO.setInvoiceStatusStr(com.gic.finance.constant.InvoiceStatusEnum.REJECT.getCode() + "");
            }
            if (InvoiceStatusEnum.COMPLETE.getCode() == invoiceStatus) {
                paramQO.setInvoiceStatusStr(com.gic.finance.constant.InvoiceStatusEnum.MAILED.getCode() + "");
            }
        }
        return ResultControllerUtils.commonPageResult(invoiceManageApiService.listInvoiceManage(paramQO),
                InvoiceListVO.class);
    }

    @RequestMapping("/query-pay-status")
    public RestResponse queryPayStatus(String orderSerialNumber, Integer payType) {
        ServiceResponse<Boolean> serviceResponse = this.billingPayInfoApiService
                .queryAndUpdatePayStatus(UserDetailUtils.getUserDetail().getEnterpriseId(), orderSerialNumber, payType);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

}
