package com.gic.enterprise.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.service.AuthCodeApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.constant.PackageHoursEnum;
import com.gic.enterprise.dto.*;
import com.gic.enterprise.qo.PageQO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.BillingEnterpriseSmsPackageApiService;
import com.gic.enterprise.service.BillingOrderApiService;
import com.gic.enterprise.service.BillingSmsRecordApiService;
import com.gic.enterprise.service.PackageApiService;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.qo.EnterpriseSmsPackageOrderQO;
import com.gic.enterprise.web.qo.EnterpriseSmsPackageQO;
import com.gic.enterprise.web.vo.PackageVO;
import com.gic.enterprise.web.vo.billing.SmsPackageDetailVO;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
public class PackageController {
    private Logger logger = LogManager.getLogger(PackageController.class);
    @Autowired
    private PackageApiService  packApiService;
    @Autowired
    private BillingOrderApiService billingOrderApiService;
    @Autowired
    private BillingEnterpriseSmsPackageApiService enterpriseSmsPackageApiService;
    @Autowired
    private BillingSmsRecordApiService billingSmsRecordApiService;
    @Autowired
    private AuthCodeApiService authCodeApiService;

    @RequestMapping("list-all-package")
    public RestResponse listAllPackage(){
        ServiceResponse<List<PackageDTO>> serviceResponse = this.packApiService.listAllPackage();
        logger.info("增值服务列表："+JSON.toJSONString(serviceResponse));
        if(serviceResponse.isSuccess()){
            return RestResponse.success(EntityUtil.changeEntityListByJSON(PackageVO.class, serviceResponse.getResult()));
        }
        return EnterpriseRestResponse.failure(serviceResponse);
    }

    @RequestMapping("list-all-sms-package")
    public RestResponse listAllSmspackage(){
        ServiceResponse<List<SmsPackageDTO>> response = this.packApiService.listAllSmsPackage();
        if(response.isSuccess()){
            return RestResponse.success(response.getResult());
        }else {
            return EnterpriseRestResponse.failure(response);
        }
    }

    @RequestMapping("page-enterprise-sms-package")
    public RestResponse pageEnterpriseSmspackage(EnterpriseSmsPackageQO qo, PageQO pageQO){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<Page<EnterpriseSmsPackageDTO>> response = this.packApiService.pageEnterpriseSmsPackage(enterpriseId, qo.getSearch(),
                qo.getStatus(), qo.getStartDate(), qo.getEndDate(), pageQO.getCurrentPage(), pageQO.getPageSize());
        if(response.isSuccess()){
            return RestResponse.success(response.getResult());
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("save-sms-package-order")
    public RestResponse saveSmspackageOrder(EnterpriseSmsPackageOrderQO qo){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        BillingOrderDTO orderDTO = new BillingOrderDTO();
        orderDTO.setCouponCardId(qo.getCoupCardId() == null ? null : Integer.valueOf(qo.getCoupCardId()+""));
        orderDTO.setEnterpriseId(enterpriseId);
        orderDTO.setOrderContent(qo.getSmsPackageName());
        orderDTO.setTotalFee(qo.getTotalFee());
        orderDTO.setTotalFeePaid(qo.getTotalFeePaid());
        SmsPackageOrderItemDTO itemDTO = new SmsPackageOrderItemDTO();
        itemDTO.setBuyCount(qo.getNum());
        itemDTO.setCode(qo.getSmsPackageCode());
        itemDTO.setCost(qo.getCost());
        itemDTO.setPackageLength(qo.getPackageHours());
        itemDTO.setSmsAmount(qo.getItemCount());
        for(PackageHoursEnum packageHoursEnum : PackageHoursEnum.values()){
            if(packageHoursEnum.getCode() == itemDTO.getPackageLength()){
                itemDTO.setPackageHours(packageHoursEnum.getMessage());
            }
        }
        ServiceResponse<Integer> response = this.billingOrderApiService.saveSmsOrderBuyNow(orderDTO, itemDTO);
        if(response.isSuccess()){
            LogUtils.createLog("套餐包购买", qo.getSmsPackageName());
            return RestResponse.success(response.getResult());
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("pay-sms-package-order")
    public RestResponse paySmsPackageOrder(Integer orderId, String authCode, Integer payType, Integer authCodeId){
        ServiceResponse response = this.authCodeApiService.validateAuthCode(authCodeId, authCode);
        if(!response.isSuccess()){
            return EnterpriseRestResponse.failure(response);
        }
        this.authCodeApiService.expireAuthCode(authCodeId);
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<BillingPayInfoDTO> serviceResponse = this.billingOrderApiService.
                paySmsPackageOrder(enterpriseId, orderId, payType, authCode, UserDetailUtils.getUserDetail().getUserInfo().getUserName());
        if(serviceResponse.isSuccess()){
            BillingOrderDTO orderDTO = this.billingOrderApiService.getOrderById(orderId).getResult();
            LogUtils.createLog("套餐包支付", orderDTO.getOrderContent());
            return RestResponse.success(serviceResponse.getResult());
        }
        return EnterpriseRestResponse.failure(serviceResponse);
    }

    @RequestMapping("smspackage-detail")
    public RestResponse smsPackageDetail(Integer id){
        ServiceResponse<BillingEnterpriseSmsPackageDTO> enterpriseSmsPackageDetail = this.enterpriseSmsPackageApiService.
                getEnterpriseSmsPackageDetail(id);
        if(enterpriseSmsPackageDetail.isSuccess()){
            BillingEnterpriseSmsPackageDTO dto = enterpriseSmsPackageDetail.getResult();
            if(dto != null){
                SmsPackageDetailVO vo = new SmsPackageDetailVO();
                vo.setStatus(dto.getStatus());
                vo.setRemainAccount(dto.getRemainAccount());
                vo.setEffectiveDate(dto.getEffectiveDate());
                ServiceResponse<BillingOrderDTO> order = this.billingOrderApiService.getOrderById(dto.getBillingOrderId());
                if(order.isSuccess()){
                    BillingOrderDTO orderDTO = order.getResult();
                    if(orderDTO != null){
                        vo.setSerialNumber(orderDTO.getSerialNumber());
                        vo.setOrderContent(orderDTO.getOrderContent());
                        vo.setOrderDate(orderDTO.getCreateTime());
                        vo.setPayType(orderDTO.getPayType());
                        SmsPackageOrderItemDTO dto1 = JSON.parseObject(orderDTO.getItemJson(), SmsPackageOrderItemDTO.class);
                        vo.setSmsPackageHours(dto1.getPackageHours());
                    }
                }
                return RestResponse.success(vo);
            }
        }
        return RestResponse.success();
    }
}
