package com.gic.enterprise.web.controller.wm;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.gic.enterprise.error.ErrorCode;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.constant.MallModeEnum;
import com.gic.enterprise.constant.WmStoreConfigTypeEnum;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.dto.wm.WmStoreConfigDTO;
import com.gic.enterprise.dto.wm.WmStoreCouponConfigDTO;
import com.gic.enterprise.qo.wm.CouponConfigQO;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.utils.DomainUtils;
import com.gic.enterprise.web.vo.MemberCardVO;
import com.gic.enterprise.web.vo.wm.*;
import com.gic.marketing.api.dto.coupon.CouponDTO;
import com.gic.marketing.api.service.coupon.MemberCouponApiService;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.weimob.api.service.WeimobMerchantService;
import com.gic.weimob.api.service.WeimobStoreSiteService;

@RestController
@RequestMapping("/wm-mall-store")
public class WmMallStoreController {
    private final static Logger LOGGER = LogManager.getLogger(WmMallStoreController.class);

    @Autowired
    private WmStoreApiService wmStoreApiService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private WeimobStoreSiteService weimobStoreSiteService;
    @Autowired
    private WeimobMerchantService weimobMerchantService;
    @Autowired
    private MemberCouponApiService memberCouponApiService;

    @RequestMapping("/save-wm-store")
    public RestResponse save(WmStoreDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Integer> result = wmStoreApiService.saveWmStore(dto);
        if (result.isSuccess()) {
            //创建微盟gic关联
            ServiceResponse<Void> saveStoreInfo = weimobMerchantService
                    .saveStoreInfo(result.getResult(), dto.getEnterpriseId(),
                            dto.getWmMainAccount(), Long.valueOf(dto.getWmPid()), Long.valueOf(dto.getWmStoreId()),
                            MallModeEnum.mapToWm(dto.getMallMode()));
            LOGGER.info("微盟商城店铺-微盟结果:{}", JSON.toJSONString(saveStoreInfo));
        }
        return OperationResultUtils.operationResult(result,
                OperationResultUtils.LOG_NEW + "微盟商城店铺", dto.getWmPidName());
    }

    @RequestMapping("/edit-wm-store")
    public RestResponse edit(WmStoreDTO dto) {
        return OperationResultUtils.operationResult(wmStoreApiService.modifyWmStore(dto),
                OperationResultUtils.LOG_EDIT + "微盟商城店铺", dto.getWmPidName());
    }

    /**
     * 店铺详情
     * @param wmMallStoreId
     * @return
     */
    @RequestMapping("/get-wm-store-detail")
    public RestResponse getDetail(Integer wmMallStoreId) {
        return ResultControllerUtils.commonResultOne(wmStoreApiService.getWmStoreByWmMallStoreId(wmMallStoreId),
                WmStoreDetailVO.class);
    }

    /**
     * 店铺列表
     * @return
     */
    @RequestMapping("/list-wm-store")
    public RestResponse listWmStore() {
        ServiceResponse<List<WmStoreDTO>> listResponse = wmStoreApiService.listWmStore(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (listResponse.isSuccess()) {
            List<WmStoreDTO> list = listResponse.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                List<WmStoreVO> voList = list.stream()
                        .map(e -> {
                            WmStoreVO vo = EntityUtil.changeEntityNew(WmStoreVO.class, e);
                            ServiceResponse<Set<String>> configSet = wmStoreApiService.getWmStoreConfigKey(e.getWmMallStoreId());
                            if (configSet.isSuccess()) {
                                Set<String> set = configSet.getResult();
                                if (set.contains(String.valueOf(WmStoreConfigTypeEnum.COUPON_CONFIG.getCode()))) {
                                    vo.setHasBindCoupon(1);
                                }
                                boolean isAllConfig = Stream.of(WmStoreConfigTypeEnum.values())
                                        .allMatch(configType -> set.contains(String.valueOf(configType.getCode())));
                                if (isAllConfig){
                                    vo.setHasCompleteConfig(1);
                                }
                                //还未配置的项目
                                vo.setHasNotConfig(Stream.of(WmStoreConfigTypeEnum.values())
                                        .filter(configItem -> !set.contains(String.valueOf(configItem.getCode())))
                                        .map(configItem -> configItem.getCode())
                                        .collect(Collectors.toList()));
                            }
                            return vo;
                        })
                        .collect(Collectors.toList());
                return RestResponse.success(voList);
            }

        }
        return RestResponse.failure(listResponse.getCode(), listResponse.getMessage());
    }

    /**
     * 获取授权链接
     * @Title: authWmStore 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/get-auth-url")
    public RestResponse getAuthUrl(Integer wmMallStoreId) {
        //
        ServiceResponse<String> response = weimobMerchantService.getAuthUrl();
        if (response.isSuccess()) {
            String authUrl = response.getResult();
            String mainHost = DomainUtils.getCurrentDomain() + "/wm-mall-store/auth-callback?wmMallStoreId=" + wmMallStoreId;
            return RestResponse.success(authUrl + "&redirect_uri=" + mainHost);
        } else {
            return RestResponse.failure(response.getCode(), response.getMessage());
        }
    }

    /**
     * 同步会员等级到微盟
     * @Title: syncMemberGradeToWm 
     * @Description:
     * @author guojuxing
     * @param memberCardId
     * @param wmMallStoreId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/sync-member-grade-to-wm")
    public RestResponse syncMemberGradeToWm(String memberCardId, Integer wmMallStoreId) {
        ServiceResponse<Void> syncMemberCardResponse = weimobMerchantService
                .saveWmMallStoreIdAndMcuAreaId(wmMallStoreId, Long.valueOf(memberCardId));
        ServiceResponse<Void> syncMemberGradeResponse = weimobMerchantService
                .saveOrUpdateCardGradeSetting(wmMallStoreId, Long.valueOf(memberCardId));
        LOGGER.info("同步会员等级-会员卡结果:{}", JSON.toJSONString(syncMemberCardResponse));
        LOGGER.info("同步会员等级-会员卡等级结果:{}", JSON.toJSONString(syncMemberGradeResponse));
        return RestResponse.success();
    }

    /**
     * 获取会员卡已同步接口
     * @Title: listMemberGrade 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/list-member-grade")
    public RestResponse listMemberGrade(Integer wmMallStoreId) {
        return ResultControllerUtils.commonResult(weimobMerchantService.getGradeList(wmMallStoreId), MemberGradeVO.class);
    }

    /**
     * 获取微盟卡券列表
     * @Title: listWmCoupon 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/list-wm-coupon")
    public RestResponse listWmCoupon(Integer wmMallStoreId) {
        ServiceResponse<WmStoreDTO> wmStoreResponse = wmStoreApiService.getWmStoreByWmMallStoreId(wmMallStoreId);
        if (wmStoreResponse.isSuccess()) {
            return ResultControllerUtils.commonResult(weimobMerchantService
                    .getWeimobCouponTemplate(Long.valueOf(wmStoreResponse.getResult().getWmPid())), WmCouponVO.class);
        }
        return RestResponse.failure(wmStoreResponse.getCode(), wmStoreResponse.getMessage());
    }

    @RequestMapping("/list-gic-coupon")
    public RestResponse listGicCoupon(Integer wmMallStoreId, String cardName) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        //获取已配置的卡券ID
        ServiceResponse<Long> mcuIdResponse = weimobMerchantService.getMcuAreaId(wmMallStoreId);
        if (!mcuIdResponse.isSuccess()) {
            return RestResponse.failure(mcuIdResponse.getCode(), mcuIdResponse.getMessage());
        }
        ServiceResponse<List<CouponDTO>> gicCouponResponse = memberCouponApiService
                .listGicCoupon(enterpriseId, mcuIdResponse.getResult(), cardName);
        return ResultControllerUtils.commonResult(gicCouponResponse, CouponVO.class);
    }

    /**
     * 授权回调
     * @param code
     */
    @RequestMapping("auth-callback")
    @ResponseBody
    public void authCallBack(String code, Integer wmMallStoreId) throws IOException {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        try {
            ServiceResponse<Void> authWmResponse = weimobMerchantService.weimobStoreAuth(enterpriseId, code);
            LOGGER.info("微盟中间件返回结果：{}", JSON.toJSONString(authWmResponse));
            if (authWmResponse.isSuccess()) {
                //更新gic侧店铺授权状态
                ServiceResponse<String> wmMallStoreResponse = wmStoreApiService.authWmStore(wmMallStoreId);
                if (!wmMallStoreResponse.isSuccess()) {
                    LOGGER.warn("企业侧微盟店铺授权结果:{}", JSON.toJSONString(wmMallStoreResponse));
                }
            }
        } catch (Exception e) {
            LOGGER.warn("微盟授权返回结果:{}", e.getMessage(), e);
        }
    }

    @RequestMapping("/list-mall-mode")
    public RestResponse listMallMode() {
        Map<String, String> result = new HashMap<>(16);
        for (MallModeEnum modeEnum : MallModeEnum.values()) {
            result.put(String.valueOf(modeEnum.getCode()), modeEnum.getMsg());
        }
        return RestResponse.success(result);
    }

    /**
     * 会员卡配置-会员卡列表
     * @Title: listMemberCard 
     * @Description:
     * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/list-member-card")
    public RestResponse listMemberCard() {
        return ResultControllerUtils.commonResult(memberCardApiService.getAll(UserDetailUtils.getUserDetail().getEnterpriseId()), MemberCardVO.class);
    }

    /**
     * 获取积分配置的开关
     * @Title: getIntegralConfig 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/get-integral-config")
    public RestResponse getIntegralConfig(Integer wmMallStoreId) {
        //获取已配置的卡券ID
        ServiceResponse<Long> mcuIdResponse = weimobMerchantService.getMcuAreaId(wmMallStoreId);
        if (!mcuIdResponse.isSuccess()) {
            return RestResponse.failure(mcuIdResponse.getCode(), mcuIdResponse.getMessage());
        }
        return ResultControllerUtils.commonResult(memberCardApiService
                .isGicCalculateIntegral(mcuIdResponse.getResult(), UserDetailUtils.getUserDetail().getEnterpriseId()));
    }

    /**
     * 编辑积分配置开关
     * @Title: editIntegralConfig 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId
     * @param integralRule gic或者erp）
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/edit-integral-config")
    public RestResponse editIntegralConfig(Integer wmMallStoreId, String integralRule) {
        //获取已配置的卡券ID
        ServiceResponse<Long> mcuIdResponse = weimobMerchantService.getMcuAreaId(wmMallStoreId);
        if (!mcuIdResponse.isSuccess()) {
            return RestResponse.failure(mcuIdResponse.getCode(), mcuIdResponse.getMessage());
        }
        return ResultControllerUtils.commonResult(memberCardApiService
                .updateIntegralRule(mcuIdResponse.getResult(), UserDetailUtils.getUserDetail().getEnterpriseId(), integralRule));
    }

    @RequestMapping("/list-wm-store-config")
    public RestResponse listWmStoreConfig() {
        Map<String, String> result = new HashMap<>(16);
        for (WmStoreConfigTypeEnum modeEnum : WmStoreConfigTypeEnum.values()) {
            if (modeEnum.isHasConfigData()) {
                result.put(String.valueOf(modeEnum.getCode()), modeEnum.getMessage());
            }
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/save-or-update-wm-store-config")
    public RestResponse saveOrUpdateWmStoreConfig(WmStoreConfigDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils
                .operationResult(wmStoreApiService.saveOrUpdateWmStoreConfig(dto),
                        "店铺配置"
                                + WmStoreConfigTypeEnum.getMessageByCode(dto.getStoreConfigType()),
                        "微盟商城");
    }

    @RequestMapping("/get-wm-store-config")
    public RestResponse getWmStoreConfig(Integer wmMallStoreId, Integer wmStoreConfigType) {
        return ResultControllerUtils
                .commonResult(wmStoreApiService.getWmStoreConfig(wmMallStoreId, wmStoreConfigType));
    }

    @RequestMapping("/bind-coupon")
    public RestResponse bindCoupon(WmStoreCouponConfigDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        Map<String, CouponDTO> gicCouponMap = getGicCoupon(dto.getWmMallStoreId());
        CouponDTO couponDTO = gicCouponMap.get(dto.getGicCouponId());
        if (couponDTO == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "gic卡券不存在");
        }
        //计算过期时间
        if (couponDTO.getCardEffectiveMode().intValue() == 0) {
            //固定日期，需要给一个过期时间
            dto.setGicCouponExpire(couponDTO.getExpireDate());
        }
        return OperationResultUtils
                .operationResult(wmStoreApiService.saveCouponConfig(dto),
                        "绑定卡券"
                                + dto.getGicCouponName() + ":" + dto.getWmCouponName(),
                        "微盟商城");
    }

    @RequestMapping("/page-coupon-config")
    public RestResponse pageCouponConfig(CouponConfigQO params) {
        return ResultControllerUtils
                .commonResult(wmStoreApiService.pageCouponConfig(params));
    }

    private Map<String, CouponDTO> getGicCoupon(Integer wmMallStoreId) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        //获取已配置的卡券ID
        ServiceResponse<Long> mcuIdResponse = weimobMerchantService.getMcuAreaId(wmMallStoreId);
        if (!mcuIdResponse.isSuccess()) {
            return new HashMap<>(2);
        }
        ServiceResponse<List<CouponDTO>> gicCouponResponse = memberCouponApiService
                .listGicCoupon(enterpriseId, mcuIdResponse.getResult(), "");
        if (gicCouponResponse.isSuccess()) {
            List<CouponDTO> list = gicCouponResponse.getResult();
            return list.stream().collect(Collectors.toMap(e -> e.getCouponId().toString(), e -> e));
        }
        return new HashMap<>(2);
    }
}
