package com.gic.enterprise.web.controller.wm;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constant.MallModeEnum;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.store.dto.StoreBusinessTimeDTO;
import com.gic.store.dto.StoreDTO;
import com.gic.store.service.StoreApiService;
import com.gic.weimob.api.dto.WeimobPhysicalStoreDTO;
import com.gic.weimob.api.dto.WeimobStoreAccountDTO;
import com.gic.weimob.api.service.WeimobStoreSiteService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * 同步门店to微盟
 * @ClassName: WmStoreSyncController 
 * @Description:  
 * @author guojuxing 
 * @date 2020/5/8 11:19 AM 
 */
@RestController
@RequestMapping("/wm-store-sync")
public class WmStoreSyncController {
    private static final Logger LOGGER = LogManager.getLogger(WmStoreSyncController.class);

    @Autowired
    private WmStoreApiService wmStoreApiService;
    @Autowired
    private WeimobStoreSiteService weimobStoreSiteService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private UserApiService userApiService;


    /**
     *
     * @param request
     * @param response
     * @throws IOException
     */
    @RequestMapping("wm-store-import-template")
    public void wmStoreImportTemplate(HttpServletRequest request, HttpServletResponse response)
            throws IOException {
        ClassPathResource res = new ClassPathResource("exceltemplate/wm_store_import_template.xls");
        String userAgent = request.getHeader("User-Agent").toLowerCase();
        String fileName = "门店同步模板.xls";
        if (userAgent.contains("firefox")) {
            byte[] bytesName = fileName.getBytes("UTF-8");
            fileName = new String(bytesName, "ISO-8859-1");
            response.setHeader("content-disposition", "attachment;fileName=" + fileName);
        } else {
            response.setHeader("content-disposition", "attachment;fileName=" + URLEncoder.encode(fileName, "UTF-8"));
        }

        InputStream is = res.getInputStream();
        OutputStream out = response.getOutputStream();
        int bytes = 0;
        byte[] bufferOut = new byte[1024];
        while ((bytes = is.read(bufferOut)) != -1) {
            out.write(bufferOut, 0, bytes);
        }

        out.close();
        is.close();
    }

    /**
     * 总店-分店同步
     * @Title: syncHeadStore 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 店铺逐渐ID
     * @param headStoreId 总店ID
     * @param storeIds  分店IDs
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("sync-head-store")
    public RestResponse syncHeadStore(Integer wmMallStoreId, Integer headStoreId, String storeIds) {
        ServiceResponse<WmStoreDTO> wmStoreResponse = wmStoreApiService.getWmStoreByWmMallStoreId(wmMallStoreId);
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if (wmStoreResponse.isSuccess()) {
            WmStoreDTO wmStoreDTO = wmStoreResponse.getResult();
            //是否智慧零售模式
            Integer mode = wmStoreDTO.getMallMode();
            if (mode.intValue() != MallModeEnum.SMART_RETAIL.getCode()) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "非" + MallModeEnum.SMART_RETAIL.getMsg() + "不能同步门店");
            }
            ServiceResponse<StoreDTO> storeResponse = storeApiService.getStoreById(enterpriseId, headStoreId);
            if (!storeResponse.isSuccess()) {
                return RestResponse.failure(storeResponse.getCode(), storeResponse.getMessage());
            }
            StoreDTO storeDTO = storeResponse.getResult();
            WeimobPhysicalStoreDTO weimobPhysicalStoreDTO = getWeimobPhsicalStore(storeDTO);
            if (weimobPhysicalStoreDTO == null) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "总店不存在或者未启用");
            }
            //分店
            List<WeimobPhysicalStoreDTO> subStoreList = new ArrayList<>();
            if (StringUtils.isNotBlank(storeIds)) {
                String[] subStoreIdArr = storeIds.split(",");
                subStoreList = Stream.of(subStoreIdArr).map(e -> {
                    ServiceResponse<StoreDTO> storeTempResponse = storeApiService.getStoreById(enterpriseId, headStoreId);
                    if (storeTempResponse.isSuccess()) {
                        return getWeimobPhsicalStore(storeTempResponse.getResult());
                    }
                    return null;
                }).collect(Collectors.toList());
            }
            ServiceResponse<Void> wmResult = weimobStoreSiteService
                    .addWeimobVirtualStore(wmMallStoreId, enterpriseId, weimobPhysicalStoreDTO, subStoreList);
            if (wmResult.isSuccess()) {
                //todo 同步导购
                return RestResponse.success();
            }
            return RestResponse.failure(wmResult.getCode(), wmResult.getMessage());
        }
        return RestResponse.failure(wmStoreResponse.getCode(), wmStoreResponse.getMessage());
    }

    private WeimobPhysicalStoreDTO getWeimobPhsicalStore(StoreDTO storeDTO) {
        if (storeDTO.getStatus().intValue() == 1) {
            WeimobPhysicalStoreDTO weimobPhysicalStoreDTO = new WeimobPhysicalStoreDTO();
            weimobPhysicalStoreDTO.setStoreId(storeDTO.getStoreId());
            weimobPhysicalStoreDTO.setStoreNumber(storeDTO.getStoreCode());
            weimobPhysicalStoreDTO.setStoreName(storeDTO.getStoreName());
            //经纬度
            if (StringUtils.isNotBlank(storeDTO.getLongitude())) {
                weimobPhysicalStoreDTO.setLongitude(new BigDecimal(storeDTO.getLongitude()));
            }
            if (StringUtils.isNotBlank(storeDTO.getLatitude())) {
                weimobPhysicalStoreDTO.setLatitude(new BigDecimal(storeDTO.getLatitude()));
            }
            //省市区名称
            String fullArea = StringUtils.isNotBlank(storeDTO.getProvinces()) ?
                    storeDTO.getProvinces().replaceAll("//", "") : "";
            weimobPhysicalStoreDTO.setAddress(fullArea + storeDTO.getAddress());
            StringBuilder businessTime = new StringBuilder();
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH:mm");
            List<StoreBusinessTimeDTO> businessTimeList = storeDTO.getBusinessTimeList();
            if (CollectionUtils.isNotEmpty(businessTimeList)) {
                businessTimeList.stream().map(e -> {
                    String businessHours = simpleDateFormat.format(e.getOpenTime()) + "-" + simpleDateFormat.format(e.getCloseTime());
                    return e.getWeekdayShow() + businessHours;
                }).forEach(businessTime::append);
            }
            weimobPhysicalStoreDTO.setBussinessHours(businessTime.toString());

            WeimobStoreAccountDTO accountDTO = new WeimobStoreAccountDTO();
            ServiceResponse<UserDTO> userResponse = userApiService.getUserByEnterpriseId(storeDTO.getEnterpriseId());
            if (userResponse.isSuccess()) {
                UserDTO userDTO = userResponse.getResult();

            }
            if (StringUtils.isNotBlank(storeDTO.getConactsPhone())) {
                weimobPhysicalStoreDTO.setStoreTel(storeDTO.getConactsPhone());
            } else {
                //
            }
            weimobPhysicalStoreDTO.setAccountVo(accountDTO);
            return weimobPhysicalStoreDTO;
        }
        return null;
    }
}
